/* ================================================================== */
/* =                      md5sumjpegdata                            = */
/* ------------------------------------------------------------------ */
/* Author/Copyright: Oliver Bandel, Berlin (Germany)                  */
/* Copyleft:         This is free Software, License: GPLv3            */
/*                                                                    */
/* No warranty on this software; use at your own risk.                */
/*                                                                    */
/* The only warranty I give, is: I have done it with best effort. :)  */
/* ------------------------------------------------------------------ */
/*                                                                    */
/* This program calculates the md5sum of the pure jpeg-picture data,  */
/* ignoring any header entry.                                         */
/*                                                                    */
/* This allows you to compare pictures of the same contents, which    */
/* were only changed in the header, not in the picture data itself.   */
/*                                                                    */
/* This might be necessary, if you want to find out, which pictures   */
/* are EQUAL in it's contents, even they may differ in the metadata.  */
/* This case might happen, if your picture-management tool imports    */
/* your pics and writes information like tags into that file.         */
/* Then you can't find the two picture files with equal data inside,  */
/* just by comparing it bytewise or with md5sum of the whole file.    */
/*                                                                    */
/* =                                                                = */
/* ================================================================== */

/* 
  $Revision: 58 $
*/

/* common stuff */
/* ------------ */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <errno.h>
#include <assert.h>


/*
jpeglib.h, jerror.h, jconfig.h, jmorecfg.h.
*/

/* jpeglib-stuff */
/* ------------- */
#include <jpeglib.h>
#include <jerror.h>
#include <jerror.h>
#include <jconfig.h>
#include <jmorecfg.h>


/* md5-stuff */
/* --------- */
#include <openssl/md5.h>








/* -------------------------------- */
/* opens a file with error-checking */
/* -------------------------------- */
FILE* Open_file_read( char* name )
{
  FILE* fp = NULL;


  /* check name-ptr on validity */
  /* -------------------------- */
  assert( name );


  if( (fp = fopen(name, "rb")) == NULL )
  {
    fprintf(stderr, "can't open %s\n", name);
    perror("can't open file!");
    exit(EXIT_FAILURE);
  }

  return fp;
}


/* ------------------------------------------ */
/* allocates one scan line (for the jpg-data) */
/* ------------------------------------------ */
JSAMPROW allocate_scanline( int output_width,  int output_components )
{
  JSAMPLE* linemem = calloc( output_width * output_components, sizeof(JSAMPLE) );
  if( linemem == NULL )
  {
    fprintf(stderr, "not enough memory for the task!\n");
    exit(EXIT_FAILURE);
  }

  return linemem;
}



/* ----------------------------------------------------- */
/* needs filename to read from, and a buffer to write to */
/* ----------------------------------------------------- */
void calculate_md5( char* filename, unsigned char* md_buf )
{
  FILE* fp = NULL;

  JSAMPROW   scanline   = NULL;  /* one scanline (pic-data) from the jpeg-file */
  struct jpeg_decompress_struct  jpg_obj;   /* decompression object (input)    */
  struct jpeg_error_mgr          jpeg_err;  /* error handler                   */

  MD5_CTX  struc;    /* Structure for calculating the MD5-sum */
  
  
  /* checking filename on validity */
  /* ----------------------------- */
  assert( filename );
  assert( md_buf );


  /* Picfile zum lesen öffnen */
  /* ------------------------ */
  fp = Open_file_read( filename ); /* erledigt schon fehlerchecks */




  /* preparing the jpeg-datastructures */
  /* --------------------------------- */
  jpg_obj.err = jpeg_std_error( &jpeg_err ); /* set this error handler for this input-oobj. */
  jpeg_create_decompress( &jpg_obj ); /* create/ninitialize the decompress-object */
  jpeg_stdio_src( &jpg_obj, fp );  /* standard source-module will be invokes as input-handler */

  jpeg_read_header(&jpg_obj, TRUE); /* read pic-infos */

  jpeg_start_decompress( &jpg_obj );

  /* MD5-Struktur vorbereiten */
  /* ------------------------ */
  MD5_Init( &struc );

  /* allocating memory for one scanline */
  /* ---------------------------------- */
  scanline =  allocate_scanline( jpg_obj.output_width,  jpg_obj.output_components );


  /* decompression-process */
  /* --------------------- */
  while (jpg_obj.output_scanline < jpg_obj.output_height)
  {
    /* read one scanline */
    /* ----------------- */
    jpeg_read_scanlines(&jpg_obj, &scanline, 1);

    /* feed scanline to MD5-hash-structure */
    /* ----------------------------------- */
    MD5_Update( &struc, scanline , jpg_obj.output_width * jpg_obj.output_components );
  }
  jpeg_finish_decompress( &jpg_obj ); /* ready with reading picture data */

  fclose(fp);  /* no further reads on the picture file */
  jpeg_destroy_decompress( &jpg_obj ); /* deallocate the jpeg-structures; we are done tih it */

  /* calc final result of the MD5-sum */
  /* -------------------------------- */
  MD5_Final( md_buf, &struc );

  return;
}











/* --------------------------------------------------- */
/* just prints out the result as hex, with leading "0" */
/* --------------------------------------------------- */
void print_hex_of_digest( unsigned char* md5_digest )
{
  unsigned long int idx = 0;

  for( idx = 0; idx < MD5_DIGEST_LENGTH; idx++ )
  {
    printf("%02x", md5_digest[ idx ] );
  }

  return;
}


/* ---- */
/* MAIN */
/* ---- */
int main( int argc, char*argv[] )
{

  unsigned char md[ MD5_DIGEST_LENGTH ];
  unsigned long int idx;

  char* filename = NULL;


  if( argc == 1 )
  {
    printf("%s: give me filenames of jpg-files as arguments and I will calculate\n", argv[0]);
    printf("               the md5sum of the picture-data for you (ignoring any headers).\n");
  }

  /* for each filename calc md5 of jpeg-picture-data and print filename and md5 */
  /* -------------------------------------------------------------------------- */
  for( idx = 1; idx < argc; idx++ )
  {
    filename = argv[idx];

    calculate_md5( filename, md );

    /* print results */
    /* ------------- */
    print_hex_of_digest( md );
    printf(" %s\n",  filename );
  }

  
  return 0;
}


